<?php
define ( "JOIN_LEFT", 1 );
define ( "ORDER_ASC", 1 );
define ( "ORDER_DESC", 2 );

class SqlTableRow 
{
var $d_values;
var $d_table;
var $d_changedValues=array();
var $d_inDB;
var $d_saved;
var $d_insertId = -1;

function SqlTableRow( &$table, $values=false, $inDB=true )
{
$this->d_table = &$table;
$this->d_values = $values;
$this->d_inDB = $inDB;
if ( $values === false )
{
$this->d_values = array();
$this->d_inDB = false;
}
$this->d_saved = $this->d_inDB;
}

function getHash()
{
return $this->d_values;
}

function getInsertId()
{
return $this->d_insertId;
}

function setValue( $key, $value )
{
$this->d_values[$key] = $value;		
$this->d_changedValues[$key] = TRUE;
$this->d_saved = false;
}

function getValue( $key )
{
return $this->d_values[$key];
}

function isValid()
{
return true;
}

function delete()
{
if ( $this->d_inDB )
{
$this->d_table->deleteRow( $this );
$this->d_inDB = false;
}
}

function save()
{
if ( $this->d_saved )
return TRUE;
$err = $this->d_table->saveRow( $this );
if ( $err === TRUE )
{
$this->d_inDB = TRUE;
$this->d_saved = TRUE;
$this->d_changedValues = array();
return TRUE;
}
return $err;
}	
}

/**
 * 
 *                                                                          
 *  
 **/

class SqlTable extends BaseObject
{
var $d_fields;
var $d_rows;
var $d_name;
var $d_DB;
var $d_selectedFields;
var $d_selectString;
var $d_primaryKeyFields = array();
//               
 var $d_debugSQL = false;


//                                                                        
 function SqlTable( $name, $selectiveFields=false, $db=false )
{
if ( !$name )
halt( 7, "SQL-Table must have a name!" );
$this->d_DB = $db;
$this->d_name = $name;
$this->d_fields = mydb_desc_table( $name, $db );
foreach ( $this->d_fields as $key=>$val )
{
if ( $val["primary_key"] )
$this->d_primaryKeyFields[] = $key;
}
if ( $selectiveFields !== false )
{
$sFields = array();
foreach ( $selectiveFields as $sField )
{
$sFields[$sField] = $this->d_fields[$sField];
}
$this->d_selectedFields = $sFields;
$this->d_selectString = $this->buildSelectString( $sFields );
}
else
{
$this->d_selectString = "*";
$this->d_selectedFields = &$this->d_fields;
}
}

function setDebug( $doDebug = TRUE )
{
$this->d_debugSQL = $doDebug;
}

function buildSelectString( $fields )
{
$keys = array_keys($fields);
return implode( ",", sqlf_field_array($keys) );
}

function transformFieldData( $field, $data )
{
return mydb_transformtodbdata( $field, $data, $this->d_DB );
}

function updateRow( &$row )
{
$first = true;
$sql = "UPDATE ". sqlf_table($this->d_name) . " SET ";

foreach ( $this->d_fields as $name=>$prop )
{
if ( isset( $row->d_values[$name] ) )
{
$data = $this->transformFieldData($prop, $row->d_values[$name]); 
if ( !$prop["primary_key"] )
{
//                        
 if ( $row->d_changedValues[$name] )
{
if ( !$first )
$sql .= ", ";
$sql .= sqlf_field($name) . "=" . (($data!==NULL)?sqlf_value($data):"NULL") . " " ;
$first = false;
}
}
else {
//                                                
 $where = "WHERE ". sqlf_field($name) . "=". sqlf_value($data); 
}
} 
}
if ( $first ) //                         
 return TRUE;
if ( !$where )
halt( 7, "SqlTable::updateRow: Table (".$this->d_name.") has no primary key!" );
$sql .= $where;

if ( $this->d_debugSQL )
echo "SqlTable::updateRow: SQL= $sql<br>\n";

$rs = query( $sql, $this->d_DB );//            
 return TRUE;
}

function insertRow( &$row )
{
$first = true;
$names = $values = "";
foreach ( $this->d_fields as $name=>$prop )
{
if ( $prop["primary_key"] )
$primaryKey = $name;

if ( isset( $row->d_values[$name] ) )
{
if ( !$first )
{
$values .= ", ";
$names .= ", ";
}
if ( !($prop["primary_key"] && $prop["auto_increment"]) )
{
$names .= sqlf_field($name);
$data = $this->transformFieldData( $prop, $row->d_values[$name] );
$values .= ($data===NULL)?"NULL":sqlf_value($data);
$first = false;
}
}
}
$sql = "INSERT INTO ". sqlf_table($this->d_name) . " ( $names ) VALUES ( $values )";
$rs = query( $sql, $this->d_DB );//            
 $row->d_insertId = $rs["iid"];
$row->d_values[$primaryKey] = $rs["iid"];
if ( $this->d_debugSQL ) echo "insertRow ($rs[iid]): '".htmlentities_utf8($sql)."'<br />";
return TRUE;
}

function saveRow( &$row )
{
if ( !$row->isValid() )
halt( 7, "tried to save invalid RowData!" );
if ( $row->d_inDB )
{
return $this->updateRow( $row );
}
else
{
return $this->insertRow( $row );
}
}


function deleteRow( &$row )
{
$first = true;
foreach ( $this->d_fields as $name=>$prop )
{
if ( !$first )
$sql .= ", ";

if ( $prop["primary_key"] )
$where = "WHERE ". sqlf_field($name) . "=". sqlf_value($this->transformFieldData($prop, $row->d_values[$name] )); 
}

if ( !$where )
halt( 7, "SqlTable::deleteRow(): Table (".$this->name.") has no primary key!" );

$sql = "DELETE FROM ". sqlf_table($this->d_name) . " $where";
$rs = query( $sql, $this->d_DB );//            
 return TRUE;
}

function buildWhereCondition( $key, $value, $comparator, $multiValueComparator ) {
if ( !is_array($key) && $key !== false )
$key = array( $key );
if ( !is_array($value) || $multiValueComparator )
$value = array( $value );
if ( $comparator === false )
$comparator = "=";

//                                       
 $sql = "";
if ( $key !== false )
{
$first = true;
foreach ( $key as $cKey=>$kValue )
{
if ( $first )
$first = false;
else
$sql .= " AND ";

if ( $multiValueComparator )
{
$mvValues = array();
if ( is_array( $value[$cKey] ) )
{
foreach ( $value[$cKey] as $mvKey=>$mvVal )
{
$mvValues[] = $this->transformFieldData( $this->d_fields[$key[$cKey]], $value[$cKey][$mvKey] );
}
$values = "(" . implode( ",", sqlf_value_array($mvValues) ) . ")";
}
else
return "";
}
else {
$values = $this->transformFieldData( $this->d_fields[$key[$cKey]], $value[$cKey] );
if ($values !== NULL) {
$values = sqlf_value($values);
}
}

if ( $values === NULL ) {
switch ( $comparator )
{
case "=":
$isNullCond = " IS NULL ";
break;
case "!=":
case "<>":
$isNullCond = " IS NOT NULL ";
break;
default:
warning( 107, "could not add correct null-value to sql-statement for comparator($comparator)!" );
$isNullCond = " IS NULL ";
}
$sql .= sqlf_field($key[$cKey]) . $isNullCond;
}
else {
$sql .= sqlf_field($key[$cKey]) . " " . (is_array($comparator)?$comparator[$cKey]:$comparator) .
" " . $values;
}
}
}

return $sql;
}

function deleteBy( $key, $value, $comparator="=", $multiValueComparator = false )
{
$whereCondition = $this->buildWhereCondition( $key, $value, $comparator, $multivalueComparator );
$sql = "DELETE FROM ".sqlf_table($this->d_name).
" WHERE $whereCondition "; //                                                                              
 
query( $sql, $this->d_DB );//            
 }

function findBy( $key=false, $value=false, $comparator="=", $orderby=false, $limit=false, $multiValueComparator = false )
{

$ret = array();

$sql = "SELECT ".$this->d_selectString." FROM ".sqlf_table($this->d_name);
//                                       
 if ( $key !== false )
{
$sql .= " WHERE ";

$where = $this->buildWhereCondition( $key, $value, $comparator, $multiValueComparator );
if ( $where ) {
$sql .= $where;
} else {
$sql .= "1=2";
}

/*                 
                                     
    
                 
                    
        
                     
     
                                
     
                         
                                     
      
                                                 
       
                                                                                                      
       
                                                      
      
         
                     
     
        
                                                                                        
     
                                                                                        
                      
    */
}
//                                   
 if ( $orderby !== false )
{
if ( !is_array( $orderby ) ) 
$sql .= " ORDER BY ".sqlf_field_with_keyword($orderby);
else {
$sql .= " ORDER BY ";
foreach ( $orderby as $col=>$way )
{
$sql .= sqlf_field($col);
if ( $way == ORDER_ASC )
$sql .= " ASC ";
else
$sql .= " DESC ";
}
}
}
//                                        
 if ( $this->d_debugSQL )
echo "findBy: $sql<br>";
//                                    
 $rs = query( $sql, $this->d_DB, $limit );//            
 foreach ( $rs["arr"] as $result )
{
$ret[] = new SqlTableRow( $this, $result );		
}
return $ret;
}

function findAll( )
{
return $this->findBy();
}
}

class SqlJoinTable 
{
var $d_tables;
var $d_fields;
var $d_conditionString;
var $d_fromString;
var $d_selectString;
var $d_debugSQL = false;

function SqlJoinTable( $tables, $conditions, $joinConditions=array() )
{
$fields = array();
$selectedFields = array();
foreach ( $tables as $table )
{
foreach ( $table->d_fields as $key=>$value )
{
$value["full_name"] =$table->d_name."_".$key; 
$fields[$table->d_name.".".$key] = $value;
}
foreach ( $table->d_selectedFields as $key=>$value )
{
$selectedFields[$table->d_name.".".$key] = $fields[$table->d_name.".".$key];
}
}
$this->d_fields = $fields;
$this->d_selectString = $this->buildSelectString( $selectedFields );
$this->d_fromString = $this->buildFromString( $tables, $joinConditions );
$this->d_tables = &$tables;
$this->d_conditionString = $this->buildConditionString($conditions);
}

function setDebug( $doDebug = TRUE )
{
$this->d_debugSQL = $doDebug;
}

function buildConditionString( $conditions )
{
if ( !is_array($conditions) )
return $conditions;
if ( count($conditions) == 0 ) 
return " 1=1 ";

$first = true;
$ret = "(";
foreach ( $conditions as $condition )
{
if ( $first )
$first = false;
else
$ret .= " AND ";
$ret .= $condition;
}
return $ret.")";
}


function buildSelectString( $fields )
{
$first = true;
$ret = "";
foreach ( $fields as $name=>$prop )
{
if ( $first )
$first = false;
else
$ret .= ",";
$ret .= sqlf_field($name) ." AS ". sqlf_field($prop["full_name"]);
}
return $ret;
}

function buildFromString( $tables, $joinConditions )
{
$first = true;
$ret = "";
foreach ( $tables as $table )
{
if ( !isset( $joinConditions[$table->d_name] ) )
{
if ( $first )
$first = false;
else
$ret .= ",";
$ret .= sqlf_table($table->d_name);
}
else
{
switch ( $joinConditions[$table->d_name]["type"] )
{
case JOIN_LEFT:
$ret .= " LEFT JOIN ".sqlf_table($table->d_name). " ON (".
$joinConditions[$table->d_name]["cond"] . ")";
break;
}
}
}
return $ret;
}

function transformFieldData( $field, $data )
{
return mydb_transformtodbdata( $field, $data, $this->d_tables[0]->d_DB );		
}

function buildResultSet( $rs )
{
$ret = array();
foreach ( $rs["arr"] as $result )
{
$subRet = array();
foreach ( $this->d_tables as $key=>$table )
{
$subResult = array();
foreach ( $table->d_selectedFields as $fName=>$prop )
{
$subResult[$fName] = $result[$table->d_name."_".$fName]; 
}
$subRet[$table->d_name] = new SqlTableRow( $this->d_tables[$key], $subResult );
}
$ret[] = $subRet;		
}
return $ret;
}

function findBy( $key, $value, $comparator="=", $orderby=false, $limit=false )
{
if ( !is_array($key) )
$key = array( $key );
if ( !is_array($value) )
$value = array( $value );

$ret = array();

$sql = "SELECT ".$this->d_selectString." FROM ".$this->d_fromString. " WHERE ".$this->d_conditionString;

foreach ( $key as $cKey=>$kValue )
{
$sql .= " AND (" . SQLTable::buildWhereCondition( $key, $value, $comparator, false ) . ")";

}

//                                   
 if ( $orderby !== false )
{
if ( !is_array( $orderby ) ) 
$sql .= " ORDER BY ".sqlf_field_with_keyword($orderby);
else {
$sql .= " ORDER BY ";
foreach ( $orderby as $col=>$way )
{
$sql .= sqlf_field($col);
if ( $way == ORDER_ASC )
$sql .= " ASC ";
else
$sql .= " DESC ";
}
}
}	
$rs = query( $sql, $this->d_DB );//            
 if ( $this->d_debugSQL )
echo "SQLJoinTable::findBy;sql: $sql<br>\n";
return $this->buildResultSet( $rs );
}
}

?>
