<?php


/**
 *                        
 *                                                                       
 *
 **/
class MiniSQLDatasource extends MiniDatasource {

var $db;

var $hasHeader;

var $cols;
var $colheader;
var $colshow;
var $colsort;
var $idcol;
var $maincol;

var $icon;

var $from;
var $where;
var $groupby;

var $sortedcol;
var $sortedway;

function MiniSQLDatasource($db = false) {
global $DB;
$this->MiniDatasource();
$this->hasHeader = false;
$this->cols = array();
$this->colheader = array();
$this->colshow = array();
$this->colsort = array();
$this->sortedcol = "";
$this->sortedway = "ASC";
$this->idcol = "";
$this->db = ($db)?$db:$DB;
$this->icon = "";
$this->maincol = "";
}

function __wakeup() {
mydb_reconnect($this->db);
}

function setDBHandle($db) {
$this->db = $db;
}

function getDBHandle() {
return $this->db;
}

function setSQL($from, $where) {
$this->setFromSQL($from);
$this->setWhereSQL($where);
}

function setFromSQL($sql) {
$this->from = $sql;
}

function getFromSQL() {
return $this->from;
}

function setWhereSQL($sql) {
$this->where = $sql;
}

function getWhereSQL() {
return $this->where;
}

function setGroupBySQL($sql) {
$this->groupby = $sql;
}

function getGroupBySQL($sql) {
return $this->groupby;
}

function showColumnHeader($val) {
$this->hasHeader = $val;
}

function setIDColumn($colname, $sql = "") {
$this->cols[$colname] = $sql;
$this->idcol = $colname;
}

function getIDColumn() {
return $this->idcol;
}

function setMainColumn($colname) {
$this->maincol = $colname;
if ($this->colsort[$colname] && !$this->sortedcol) {
$this->setSortColumn($colname);
}
}

function getMainColumn() {
return $this->maincol;
}

function setColumn($colname, $sql = "", $header = "", $visible = false, $sortable = false) {
$this->cols[$colname] = $sql;
$this->colheader[$colname] = $header;
$this->colshow[$colname] = $visible;
$this->colsort[$colname] = $sortable;
}

function unsetColumn($colname) {
unset($this->cols[$colname]);
unset($this->colheader[$colname]);
unset($this->colshow[$colname]);
unset($this->colsort[$colname]);
if ($colname == $this->sortedcol) {
$this->sortedcol = "";
$this->sortedway = "";
}
if ($colname == $this->idcol) {
$this->idcol = "";
}
if ($colname == $this->maincol) {
$this->maincol = "";
}
}

function clearColumns() {
$this->cols = array();
$this->colheader = array();
$this->colshow = array();
$this->colsort = array();
$this->sortedcol = "";
$this->sortedway = "";
$this->idcol = "";
$this->maincol = "";
}

function setIcon($icon) {
$this->icon = $icon;
}

function unsetIcon() {
$this->icon = "";
}

function setSortColumn($column, $way = "ASC") {
if ($this->colsort[$column]) {
$this->sortedcol = $column;
$this->sortedway = $way;
}
}

function getResultSet( $fromIdx, $toIdx, &$count, $search = "", $fetchAll = false ) {

$sqlcols = "";
$sqlfrom = "";
$sqlwhere = "";
$sqlorder = "";
$limit = false;

reset($this->cols);
while (list($key,$val) = each($this->cols)) {
if ($sqlcols) $sqlcols .= ", ";
if (!$val || $key == $val) $sqlcols .= sqlf_field_or_value($key);
else $sqlcols .= sqlf_field_or_value($val)." AS ".sqlf_field($key);
}


if ($this->from) $sqlfrom = "FROM ".$this->_parseSQL($this->from);
if ($this->where) $sqlwhere = "WHERE ".$this->_parseSQL($this->where);
if ($this->sortedcol) $sqlorder = "ORDER BY ".sqlf_field($this->sortedcol)." ".((strtolower($this->sortedway) == "desc")?"DESC":"ASC");

if (!$fetchAll) {
$limit = array( $fromIdx, $toIdx-$fromIdx + 1);

$sql = "SELECT COUNT(*) AS c $sqlfrom $sqlwhere";
$rs = query($sql, $this->db);//            
 $count = $rs["arr"][0]["c"];
}

$sql = "SELECT $sqlcols $sqlfrom $sqlwhere $sqlorder";
$rs = query($sql, false, $limit, $this->db);//            
 
if ($fetchAll) {
$count = $rs["num"];
}

$r = array();
while (list($key,$val) = each($rs["arr"])) {
if ($this->icon) $val["icon"] = img($this->icon);
/**                                                                             */
if ($this->maincol) $val[$this->maincol] = "<a href=\"".$this->buildLink($val[$this->idcol])."\">".$val[$this->maincol]."</a>";
$r[] = $val;
}

return $r;
}

function isSearchable() {
return false;
}

function isPreSelected( $id ) {
return false;
}

function isAllPreSelected( ) {
return false;
}

function getKey()  {
return "minisqlds_".$this->getFolderID()."_".$this->getFolderType();
}

function setSort( $column, $way ) {
if ($this->colsort[$column]) {
$this->sortedcol = $column;
$this->sortedway = $way;
}
}

function getCurrentSort( &$column, &$way ) {
if ($this->sortedcol) {
$column = $this->sortedcol;
$way = (strtolower($this->sortedway == "desc"))?"DESC":"ASC";
return true;
} else {
return false;
}
}

function getSortWays( $column ) {
global $ICO_SORT_ASC, $ICO_SORT_DESC;

if ($this->colsort[$column]) {
return array( array( "way" => "ASC", "icon" => $ICO_SORT_ASC), array ( "way" => "DESC", "icon" => $ICO_SORT_DESC) );
} else {
return array();
}
}

function getColumns() {
$rs = array();

if ($this->icon) $rs[] = "icon";

reset($this->cols);
while (list($key,$val) = each($this->cols)) {
if ($this->colshow[$key]) $rs[] = $key;
}

return $rs;
}

function hasColumnHeaders() {
return $this->hasHeader;
}

function getColumnHeader( $column ) {
return $this->colheader[$column];
}

function getColumnAttribs( $column ) {
return " nowrap";
}

function getRowHelp( $rowKey, $rowData ) {
return "";
}

function _parseSQL($sql) {
$sql = str_replace("\$folder_id",sqlf_tl_num($this->getFolderID()),$sql);
$sql = str_replace("\$folder_type",sqlf_tl_num($this->getFolderType()),$sql);
$sql = str_replace("\$id",sqlf_tl_num($this->getID()),$sql);

return $sql;
}
}
?>